@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4; coding: utf-8 -*- 
  *
- * Copyright © 2007 Björn Lindqvist <bjourne@gmail.com>
+ * Copyright © 2007-2010 Björn Lindqvist <bjourne@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -37,6 +37,8 @@
 /*************************************************************/
 /**
  * gtk_image_tool_dragger_is_draggable:
+ * @returns: %TRUE if the view can be dragged when the cursor is at
+ *   the position specified by @x and @y.
  *
  * Returns %TRUE if the view can be dragged when the cursor is at the
  * position specified by @x and @y.
@@ -128,7 +130,7 @@ motion_notify (GtkIImageTool  *tool,
     mouse_handler->drag_base_x = mouse_handler->drag_ofs_x;
     mouse_handler->drag_base_y = mouse_handler->drag_ofs_y;
     
-    return FALSE;
+    return TRUE;
 }
 
 static void
