@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4; coding: utf-8 -*- 
  *
- * Copyright © 2007 Björn Lindqvist <bjourne@gmail.com>
+ * Copyright © 2007-2008 Björn Lindqvist <bjourne@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -405,18 +405,15 @@ button_press (GtkIImageTool  *tool,
         return FALSE;
     if (selector->hotspot_type == HOTSPOT_OUTSIDE)
     {
-        GdkRectangle viewport, image_rect;
-        gtk_image_view_get_viewport (selector->view, &viewport);
-        gtk_image_view_get_draw_rect (selector->view, &image_rect);
+        GdkRectangle widget_rect = {ev->x, ev->y, 1, 1};
+        GdkRectangle image_rect;
         
-        int x = viewport.x - image_rect.x + ev->x;
-        int y = viewport.y - image_rect.y + ev->y;
-        gdouble zoom = gtk_image_view_get_zoom (selector->view);
-        x = (int) ((gdouble) x / zoom);
-        y = (int) ((gdouble) y / zoom);
-
-        GdkRectangle new_sel = {x, y, 0, 0};
-        gtk_image_tool_selector_set_selection (selector, &new_sel);
+        gtk_image_view_widget_to_image_rect (selector->view,
+                                             &widget_rect,
+                                             &image_rect);
+        image_rect.width = 0;
+        image_rect.height = 0;
+        gtk_image_tool_selector_set_selection (selector, &image_rect);
         selector->hotspot_type = HOTSPOT_RESIZE_SOUTH_EAST;
     }
     
@@ -495,7 +492,7 @@ motion_notify (GtkIImageTool  *tool,
         g_source_remove (selector->timer_id);
         selector->timer_id = 0;
     }
-    return FALSE;
+    return TRUE;
 }
 
 static void
