@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4; coding: utf-8 -*- 
  *
- * Copyright © 2007 Björn Lindqvist <bjourne@gmail.com>
+ * Copyright © 2007-2009 Björn Lindqvist <bjourne@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -104,6 +104,18 @@
  *       </tbody>          
  *     </table> 
  *   </para>
+ *   <para>
+ *     Note that the widget must be given keyboard focus otherwise the
+ *     keybindings wont work:
+ *     <informalexample>
+ *       <programlisting>
+ *         // Give GtkImageView keyboard focus.
+ *         gtk_widget_grab_focus (GTK_WIDGET (view));
+ *       </programlisting>  
+ *     </informalexample>
+ *   </para>
+ * </refsect2>
+ * <refsect2>
  *   <title>Mouse actions</title>
  *   <para>
  *     When focused, GtkImageView responds to the following mouse
@@ -124,6 +136,8 @@
  *         </tr>  
  *       </tbody>
  *   </table>
+ * </refsect2>
+ * <refsect2>
  *   <title>Coordinate systems</title>
  *   <para>
  *     Operations on GtkImageView are executed in three different
@@ -156,6 +170,47 @@
  *       system is simply the image coordinate system scaled.
  *     </listitem>
  *   </itemizedlist>
+ *   <para>
+ *     For library users, the most practical coordinate systems are
+ *     the image and widget coordinate systems. To convert a widget
+ *     coordinate to image, use the following algorithm.
+ *     <example><title>Conversion from widget to image space</title>
+ *       <programlisting>
+ *                                     Aw - Iw*z
+ *                          Ox - max ( --------- , 0) + Ex
+ *                                         2
+ *         to_image_x(Ex) = ------------------------------
+ *                                        z  
+ *       </programlisting>
+ *     </example>
+ *     The coordinates y-axis works the same way. Conversion the other
+ *     way works like this:
+ *     <example><title>Conversion from image to widget space</title>
+ *       <programlisting>
+ *                                        Aw - Iw*z
+ *         to_widget_x(Ix) = z*Ix + max ( --------- , 0 ) - Ox
+ *                                            2
+ *       </programlisting>
+ *     </example>
+ *   </para>
+ *   <para>
+ *     The variables in the equations are:
+ *   </para>
+ *   <itemizedlist>
+ *     <listitem><emphasis>Ox, Oy</emphasis> : The views offset in zoom space
+ *     coordinates. See gtk_image_view_get_viewport().</listitem>
+ *     <listitem><emphasis>Aw, Ah</emphasis> : Widget allocation.</listitem>
+ *     <listitem><emphasis>Iw, Ih</emphasis> : Width and height of image.</listitem>
+ *     <listitem><emphasis>z</emphasis> : The views zoom factor. See
+ *     gtk_image_view_get_zoom().</listitem>
+ *   </itemizedlist>
+ *   <para>
+ *     For convenience sake, these algorithms are implemented by
+ *     gtk_image_view_widget_to_image_rect() and
+ *     gtk_image_view_image_to_widget_rect().
+ *   </para>
+ * </refsect2>
+ * <refsect2>
  *   <title>Settings</title>
  *   <para>
  *     GtkImageView has a few settings that can be configured by users of
@@ -253,8 +308,8 @@
  *   </para>  
  * </refsect1>
  **/
-
 #include <gdk/gdkkeysyms.h>
+#include <math.h>
 #include <stdlib.h>
 
 #include "cursors.h"
@@ -344,7 +399,7 @@ gtk_image_view_get_zoomed_size (GtkImageView *view)
 }
 
 static void
-gtk_image_view_clamp_offset (GtkImageView *view, gdouble *x, gdouble *y)
+gtk_image_view_clamp_offset (GtkImageView *view, int *x, int *y)
 {
     Size alloc = gtk_image_view_get_allocated_size (view);
     Size zoomed = gtk_image_view_get_zoomed_size (view);
@@ -423,14 +478,19 @@ gtk_image_view_set_zoom_with_center (GtkImageView *view,
     gdouble zoom_ratio = zoom / view->zoom;
     Size alloc = gtk_image_view_get_allocated_size (view);
 
-    gdouble offset_x, offset_y;
-    offset_x = (view->offset_x + center_x) * zoom_ratio - alloc.width / 2.0;
-    offset_y = (view->offset_y + center_y) * zoom_ratio - alloc.height / 2.0;
+    int offset_x, offset_y;
+
+    offset_x = (int) round ((gdouble) ((gdouble)view->offset_x + center_x) * zoom_ratio -
+                            (gdouble) alloc.width / 2.0);
+    offset_y = (int) round ((gdouble) ((gdouble)view->offset_y + center_y) * zoom_ratio -
+                            (gdouble) alloc.height / 2.0);
     view->zoom = zoom;
 
     gtk_image_view_clamp_offset (view, &offset_x, &offset_y);
+
     view->offset_x = offset_x;
     view->offset_y = offset_y;
+
     gtk_image_view_update_cursor (view);
 
     if (!is_allocating)
@@ -471,7 +531,10 @@ gtk_image_view_zoom_to_fit (GtkImageView *view,
     gdouble ratio_y = (gdouble) alloc.height / img.height;
 
     gdouble zoom = MIN (ratio_y, ratio_x);
-    zoom = MIN (zoom, 1.0);
+
+    // Disallow to small zoom factors, they eat up all memory because
+    // the filter matrices becomes to large. See #80925.
+    zoom = CLAMP (zoom, gtk_zooms_get_min_zoom (), 1.0);
 
     gtk_image_view_set_zoom_no_center (view, zoom, is_allocating);
 }
@@ -518,17 +581,11 @@ gtk_image_view_repaint_area (GtkImageView *view,
     if (!paint_rect->width || !paint_rect->height)
         return FALSE;
 
-    // This is a workaround for #80925. If the image area gets to
-    // small, then gdk-pixbuf's scale matrix becomes ridiculously
-    // large which causes noticable and wasteful cpu spikes.
-    Size alloc = gtk_image_view_get_allocated_size (view);
-    if (alloc.width <= 1 || alloc.height <= 1)
-        return FALSE;
-    
     view->is_rendering = TRUE;
     
     // Image area is the area on the widget occupied by the pixbuf. 
     GdkRectangle image_area;
+    Size alloc = gtk_image_view_get_allocated_size (view);
     gtk_image_view_get_draw_rect (view, &image_area);
     if (image_area.x > 0 ||
         image_area.y > 0 ||
@@ -556,8 +613,9 @@ gtk_image_view_repaint_area (GtkImageView *view,
         GdkInterpType interp = view->interp;
         if (view->zoom == 1.0)
             interp = GDK_INTERP_NEAREST;
-        int src_x = view->offset_x + paint_area.x - image_area.x;
-        int src_y = view->offset_y + paint_area.y - image_area.y;
+
+        int src_x = view->offset_x + paint_area.x - image_area.x; 
+        int src_y = view->offset_y + paint_area.y - image_area.y; 
 
         GdkPixbufDrawOpts opts = {
             view->zoom,
@@ -684,16 +742,11 @@ gtk_image_view_fast_scroll (GtkImageView *view,
  **/
 static void
 gtk_image_view_scroll_to (GtkImageView *view,
-                          gdouble       offset_x,
-                          gdouble       offset_y,
+                          int           offset_x,
+                          int           offset_y,
                           gboolean      set_adjustments,
                           gboolean      invalidate)
 {
-    /* The offset needs to be truncated, otherwise weird rounding
-       errors occur. */
-    offset_x = (int) offset_x;
-    offset_y = (int) offset_y;
-
     gtk_image_view_clamp_offset (view, &offset_x, &offset_y);
 
     int delta_x = offset_x - view->offset_x;
@@ -876,7 +929,7 @@ static gboolean
 gtk_image_view_hadj_changed_cb (GtkObject    *adj,
                                 GtkImageView *view)
 {
-    gdouble offset_x = GTK_ADJUSTMENT (adj)->value;
+    int offset_x = GTK_ADJUSTMENT (adj)->value;
     gtk_image_view_scroll_to (view, offset_x, view->offset_y, FALSE, FALSE);
     return FALSE;
 }
@@ -896,10 +949,6 @@ gtk_image_view_scroll_event (GtkWidget      *widget,
 {
     GtkImageView *view = GTK_IMAGE_VIEW (widget);
 
-    // Horizontal scroll events will be handled in the future.
-    g_assert (ev->direction == GDK_SCROLL_UP ||
-              ev->direction == GDK_SCROLL_DOWN);
-
     if (!(ev->state & GDK_CONTROL_MASK))
     {
         g_signal_emit (G_OBJECT (widget),
@@ -907,9 +956,12 @@ gtk_image_view_scroll_event (GtkWidget      *widget,
                        0, ev->direction);
         return TRUE;
     }
-    
+
+    // Horizontal scroll left is equivalent to scroll up and right is
+    // like scroll down. No idea if that is correct -- I have no input
+    // device that can do horizontal scrolls.
     gdouble zoom = gtk_zooms_get_zoom_out (view->zoom);
-    if (ev->direction == GDK_SCROLL_UP)
+    if (ev->direction == GDK_SCROLL_UP || ev->direction == GDK_SCROLL_LEFT)
         zoom = gtk_zooms_get_zoom_in (view->zoom);
     gtk_image_view_set_zoom_with_center (view, zoom, ev->x, ev->y, FALSE);
     
@@ -961,8 +1013,8 @@ gtk_image_view_init (GtkImageView *view)
     view->fitting = TRUE;
     view->pixbuf = NULL;
     view->zoom = 1.0;
-    view->offset_x = 0.0;
-    view->offset_y = 0.0;
+    view->offset_x = 0;
+    view->offset_y = 0;
     view->is_rendering = FALSE;
     view->show_frame = TRUE;
     view->show_cursor = TRUE;
@@ -1078,7 +1130,7 @@ gtk_image_view_init_signals (GtkImageViewClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__BOOLEAN,
                       G_TYPE_NONE,
-                      1, G_TYPE_INT);
+                      1, G_TYPE_BOOLEAN);
     /**
      * GtkImageView::scroll:
      * @view: The #GtkImageView that received the signal.
@@ -1101,8 +1153,8 @@ gtk_image_view_init_signals (GtkImageViewClass *klass)
     /**
      * GtkImageView::mouse-wheel-scroll:
      * @view: The #GtkImageView that emitted the signal.
-     * @direction: The direction of the scroll; either %GDK_SCROLL_UP
-     *      or %GDK_SCROLL_DOWN.
+     * @direction: The direction of the scroll; %GDK_SCROLL_UP,
+     *   %GDK_SCROLL_DOWN, %GDK_SCROLL_LEFT or %GDK_SCROLL_RIGHT.
      *
      * The ::mouse-wheel-scroll signal is emitted when the mouse wheel
      * is scrolled on the view and %GTK_CONTROL_MASK is
@@ -1382,7 +1434,8 @@ gtk_image_view_new (void)
  *
  * Fills in the rectangle with the current viewport. If pixbuf is
  * %NULL, there is no viewport, @rect is left untouched and %FALSE is
- * returned.
+ * returned. If the view is not allocated, the rectangles coordinates
+ * are set to the views offset and its width and height to zero.
  *
  * The current viewport is defined as the rectangle, in zoomspace
  * coordinates as the area of the loaded pixbuf the #GtkImageView is
@@ -1410,8 +1463,7 @@ gtk_image_view_get_viewport (GtkImageView *view,
  * @view: a #GtkImageView
  * @rect: a #GdkRectangle to fill in with the area of the widget in
  *   which the pixbuf is drawn.
- * @returns: %TRUE if the view is allocated and has a pixbuf, %FALSE
- *   otherwise.
+ * @returns: %TRUE if the view has a pixbuf, %FALSE otherwise.
  *
  * Get the rectangle in the widget where the pixbuf is painted. 
  *
@@ -1420,6 +1472,9 @@ gtk_image_view_get_viewport (GtkImageView *view,
  * will be drawn centered on the widget. @rect will then be
  * (25,25)-[50,50].
  *
+ * If the view is not allocated, then the rectangle will be set to
+ * (0,0)-[0,0] and %TRUE is returned.
+ *
  * This method is useful when converting from widget to image or zoom
  * space coordinates.
  **/
@@ -1441,6 +1496,47 @@ gtk_image_view_get_draw_rect (GtkImageView *view,
     return TRUE;
 }
 
+/**
+ * gtk_image_view_widget_to_image_rect:
+ * @view: a #GtkImageView
+ * @rect_in: a #GdkRectangle in widget space coordinates to convert
+ * @rect_out: a #GdkRectangle to fill in with the image space
+ *   coordinates
+ * @returns: %TRUE if the conversion was successful, %FALSE otherwise
+ *
+ * Convert a rectangle in widget space coordinates to image space
+ * coordinates. If the view is not realized, or if it contains no
+ * pixbuf, then the conversion was unsuccessful. %FALSE is then
+ * returned and @rect_out is left unmodified.
+ *
+ * The size of the output rectangle is rounded up so if the widget
+ * rectangle crosses a pixel, that pixel is included.
+ *
+ * Note that the resulting rectangle is not clipped which means that
+ * its area may lay outside the pixbuf.
+ **/
+gboolean
+gtk_image_view_widget_to_image_rect (GtkImageView *view,
+                                     GdkRectangle *rect_in,
+                                     GdkRectangle *rect_out)
+{
+    GdkRectangle image_rect, viewport = {0};
+    if (!gtk_image_view_get_draw_rect (view, &image_rect))
+        return FALSE;
+
+    gtk_image_view_get_viewport (view, &viewport);
+
+    int x = viewport.x - image_rect.x + rect_in->x;
+    int y = viewport.y - image_rect.y + rect_in->y;
+    gdouble zoom = gtk_image_view_get_zoom (view);
+    rect_out->x = (int) ((gdouble) x / zoom);
+    rect_out->y = (int) ((gdouble) y / zoom);
+
+    rect_out->width = ceil ((gdouble) rect_in->width / zoom);
+    rect_out->height = ceil ((gdouble) rect_in->height / zoom);
+    return TRUE;
+}
+
 /**
  * gtk_image_view_image_to_widget_rect:
  * @view: a #GtkImageView
@@ -1454,6 +1550,10 @@ gtk_image_view_get_draw_rect (GtkImageView *view,
  * pixbuf, then the conversion was unsuccessful, %FALSE is returned
  * and @rect_out is left unmodified.
  *
+ * The size of @rect_out is rounded up. For example, if the zoom
+ * factor is 0.25 and the width of the input rectangle is 2, then its
+ * with in widget space coordinates is 0.5 which is rounded up to 1.
+ *
  * Note that this function may return a rectangle that is not visible
  * on the widget.
  **/
@@ -1466,11 +1566,11 @@ gtk_image_view_image_to_widget_rect (GtkImageView *view,
     GdkRectangle zoom_rect = {
         rect_in->x * zoom,
         rect_in->y * zoom,
-        rect_in->width * zoom,
-        rect_in->height * zoom
+        ceil (rect_in->width * zoom),
+        ceil (rect_in->height * zoom)
     };
 
-    GdkRectangle image_rect, viewport;
+    GdkRectangle image_rect, viewport = {0};
     if (!gtk_image_view_get_draw_rect (view, &image_rect))
         return FALSE;
     
@@ -1728,6 +1828,8 @@ gtk_image_view_get_zoom (GtkImageView *view)
  *
  * Fitting is always disabled after this method has run. The
  * ::zoom-changed signal is unconditionally emitted.
+ *
+ * The default value is 1.0.
  **/
 void
 gtk_image_view_set_zoom (GtkImageView *view,
@@ -1923,6 +2025,9 @@ gtk_image_view_set_tool (GtkImageView  *view,
  * gtk_image_view_get_tool:
  * @view: A #GtkImageView
  * @returns: The currently bound image tool
+ *
+ * Gets the image tool #GtkImageView uses for rendering and handling
+ * input events.
  **/
 GtkIImageTool *
 gtk_image_view_get_tool (GtkImageView *view)
@@ -1991,8 +2096,8 @@ gtk_image_view_zoom_out (GtkImageView *view)
  * See also gtk_image_view_set_pixbuf().
  **/
 void
-gtk_image_view_damage_pixels(GtkImageView *view,
-                             GdkRectangle *rect)
+gtk_image_view_damage_pixels (GtkImageView *view,
+                              GdkRectangle *rect)
 {
     g_signal_emit (G_OBJECT (view),
                    gtk_image_view_signals[PIXBUF_CHANGED], 0);
