@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4; coding: utf-8 -*- 
  *
- * Copyright © 2007 Björn Lindqvist <bjourne@gmail.com>
+ * Copyright © 2007-2009 Björn Lindqvist <bjourne@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -24,7 +24,6 @@
 #include <gtk/gtk.h>
 
 #include "gtkiimagetool.h"
-#include "utils.h"
 
 G_BEGIN_DECLS
 
@@ -83,8 +82,8 @@ struct _GtkImageView
     gdouble          zoom;
     /* Offset in zoom space coordinates of the image area in the
        widget. */
-    gdouble          offset_x;
-    double           offset_y;
+    int              offset_x;
+    int              offset_y;
     gboolean         show_frame;
     gboolean         show_cursor;
     GdkCursor       *void_cursor;
@@ -136,6 +135,9 @@ gboolean      gtk_image_view_get_draw_rect   (GtkImageView    *view,
 void          gtk_image_view_get_check_colors (GtkImageView  *view,
                                                int           *check_color1,
                                                int           *check_color2);
+gboolean      gtk_image_view_widget_to_image_rect (GtkImageView *view,
+                                                   GdkRectangle *rect_in,
+                                                   GdkRectangle *rect_out);
 gboolean      gtk_image_view_image_to_widget_rect (GtkImageView *view,
                                                    GdkRectangle *rect_in,
                                                    GdkRectangle *rect_out);
